@@ -295,6 +295,7 @@ AcousticStudy <- function(id=NULL,
     }
     ancillary$version <- list(R = R.version.string,
                               PAMpal = packageVersion('PAMpal'))
+    ancillary$processDate <- Sys.time()
     new('AcousticStudy', id=id, events=events, files=fileTemp, gps=gps,pps=pps,
         settings=settings, effort=effort, models=models, ancillary=ancillary)
 }
@@ -314,6 +315,6 @@ is.AcousticStudy <- function(x) {
 setMethod('show', 'AcousticStudy',
           function(object) {
               cat('AcousticStudy object named ', id(object), ' with ',
-                  length(events(object)), ' AcousticEvents.', sep='')
+                  length(events(object)), ' AcousticEvents.\n', sep='')
           }
 )
